/*
 * ContextEventSample.java
 *
 * This sample java monitor listens for ContextEvent instances. These instances
 * will contain Context instances that correspond to real MonitorScript contexts.
 * This java monitor will enqueue SimpleEvent instances to these contexts.
 *
 * $Copyright(c) 2010 Progress Software Corporation. All Rights Reserved.$ 
 * $Copyright (c) 2013,2015 Software AG, Darmstadt, Germany and/or Software AG USA Inc., Reston, VA, USA, and/or its subsidiaries and/or its affiliates and/or their licensors.$ 
 * Use, reproduction, transfer, publication or disclosure is prohibited except as specifically provided for in your License Agreement with Software AG 
 */

import java.util.List;
import java.util.ArrayList;
import com.apama.jmon.*;
import com.apama.jmon.annotation.*;

@Application(name = "Context Event Sample",
			 author = "My Name",
			 version = "1.0",
			 company = "My Company",
			 description = "Context Event java monitor sample", 
			 classpath = ""
			 )

@MonitorType(description = "A simple java monitor for demonstrating the use of contexts")
public class ContextEventSample implements Monitor, MatchListener {
	private static final long serialVersionUID = 11L;
	/**
	 * This list will store incoming contexts
	 */
	List<Context> ctxList;
	int ctxCount;

	/**
	* No argument constructor used by the jmon framework on
	* application loading
	*/
	public ContextEventSample() {
		// expecting to receive three contexts
		ctxCount = 3;
		ctxList = new ArrayList<Context>(ctxCount);
	}


	/**
	* Implementation of the Monitor interface onLoad method. Sets up
	* an event expression which looks for all ContextEvents.
	*/
	public void onLoad() {
		EventExpression eventExpr = new EventExpression("all ContextEvent(*): e");
		eventExpr.addMatchListener(this);
	}


	/**
	* Implementation of the MatchListener interface match method. Extracts the
	* event that caused the event expression to trigger and enqueues an instance
	* of SimpleEvent to the retrieved context (which is set up in MonitorScript).
	*/
	public void match(MatchEvent event) {
		ContextEvent ce = ((ContextEvent)event.getMatchingEvents().get("e"));
		Context ctx = ce.ctx;
		ctxList.add(ctx);

		// enqueue a SimpleEvent instance to the retrieved context
		(new SimpleEvent("JMon->" + ctx.toString())).enqueueTo(ctx);

		if (ctxList.size() == ctxCount) {
			// enqueue a SimpleEvent instance to all contexts
			(new SimpleEvent("JMon->[all contexts]")).enqueueTo(ctxList);
		}
	}
}
