# Sample PySys testcase
# Copyright (c) 2015-2016, 2018 Software AG, Darmstadt, Germany and/or Software AG USA Inc., Reston, VA, USA, and/or its subsidiaries and/or its affiliates and/or their licensors. 
# Use, reproduction, transfer, publication or disclosure is prohibited except as specifically provided for in your License Agreement with Software AG 

from pysys.constants import *
from apama.basetest import ApamaBaseTest
from apama.correlator import CorrelatorHelper

class PySysTest(ApamaBaseTest):

	def execute(self):
		# create the correlator helper, start the correlator 
		correlator = CorrelatorHelper(self)
		correlator.start(logfile='correlator.log')
		
		# enable application event logging
		correlator.applicationEventLogging(enable=True)
		
		# set the application log file and verbosity
		correlator.setApplicationLogFile('application.log')
		correlator.setApplicationLogLevel('DEBUG')
				
		# switch profiling on
		correlator.profilingOn()
		
		# inject the simple.mon monitor (directory defaults to the testcase input)
		correlator.injectEPL(filenames=['simple.mon'])
		
		# wait a few seconds and then get the profiling information
		self.waitForSignal('application.log', expr="Number of events received", condition=">100")
		correlator.profilingGet('profile.log')
		
		# get a dump of the stringified state
		correlator.toStringAll('tostringall.log')
		
	def validate(self):
		self.addOutcome(NOTVERIFIED, 'Please manually review the profile and toStringAll output')
